@@ -425,7 +425,7 @@ public class FastLeaderElection implements Election {
                                         self.getPeerState(),
                                         response.sid,
                                         v.getPeerEpoch(),
-                                        qv.toString().getBytes());
+                                        qv.toString().getBytes(UTF_8));
                                     sendqueue.offer(notmsg);
                                 }
                             } else {
@@ -461,7 +461,7 @@ public class FastLeaderElection implements Election {
                                         self.getPeerState(),
                                         response.sid,
                                         current.getPeerEpoch(),
-                                        qv.toString().getBytes());
+                                        qv.toString().getBytes(UTF_8));
                                     sendqueue.offer(notmsg);
                                 }
                             }
